@@ -5,11 +5,12 @@ from numba import types, cgutils
 
 
 class PyCallWrapper(object):
-    def __init__(self, context, module, func, fndesc):
+    def __init__(self, context, module, func, fndesc, exceptions):
         self.context = context
         self.module = module
         self.func = func
         self.fndesc = fndesc
+        self.exceptions = exceptions
 
     def build(self):
         wrapname = "wrapper.%s" % self.func.name
@@ -20,6 +21,8 @@ class PyCallWrapper(object):
 
         builder = Builder.new(wrapper.append_basic_block('entry'))
 
+        # builder = cgutils.VerboseProxy(builder)
+
         _, args, kws = wrapper.args
 
         api = self.context.get_python_api(builder)
@@ -41,16 +44,25 @@ class PyCallWrapper(object):
             builder.ret(api.get_null_object())
 
         innerargs = []
+        cleanups = []
         for obj, ty in zip(objs, self.fndesc.argtypes):
             #api.context.debug_print(builder, "%s -> %s" % (obj, ty))
             #api.print_object(builder.load(obj))
-            val = api.to_native_arg(builder.load(obj), ty)
+            val, dtor = api.to_native_arg(builder.load(obj), ty)
             innerargs.append(val)
+            cleanups.append(dtor)
 
         status, res = self.context.call_function(builder, self.func,
+                                                 self.fndesc.restype,
                                                  self.fndesc.argtypes,
                                                  innerargs)
 
+        # Do clean up
+        for dtor in cleanups:
+            dtor()
+
+        # Determine return status
+
         with cgutils.if_likely(builder, status.ok):
             with cgutils.ifthen(builder, status.none):
                 api.return_none()
@@ -59,13 +71,33 @@ class PyCallWrapper(object):
             builder.ret(retval)
 
         with cgutils.ifthen(builder, builder.not_(status.exc)):
+            # !ok && !exc
             # User exception raised
-            # TODO we will just raise a RuntimeError for now.
-            api.raise_native_error("error in native function: %s" %
-                                   self.fndesc.mangled_name)
+            self.make_exception_switch(api, builder, status.code)
 
+        # !ok && exc
         builder.ret(api.get_null_object())
 
+    def make_exception_switch(self, api, builder, code):
+        """Handle user defined exceptions.
+        Build a switch to check which exception class was raised.
+        """
+        nexc = len(self.exceptions)
+        elseblk = cgutils.append_basic_block(builder,
+                                             ".invalid.user.exception")
+        swt = builder.switch(code, elseblk, n=nexc)
+        for num, exc in self.exceptions.items():
+            bb = cgutils.append_basic_block(builder,
+                                            ".user.exception.%d" % num)
+            swt.add_case(Constant.int(code.type, num), bb)
+            builder.position_at_end(bb)
+            api.raise_exception(exc, exc)
+            builder.ret(api.get_null_object())
+
+        builder.position_at_end(elseblk)
+        msg = "error in native function: %s" % self.fndesc.mangled_name
+        api.raise_native_error(msg)
+
     def make_const_string(self, string):
         return self.context.insert_const_string(self.module, string)
 
