@@ -8,8 +8,9 @@ import numpy
 from numba import types, utils, cgutils, typing, numpy_support, errcode
 from numba.pythonapi import PythonAPI
 from numba.targets.imputils import (user_function, python_attr_impl,
-                                    builtin_registry, impl_attribute)
-from numba.targets import builtins
+                                    builtin_registry, impl_attribute,
+                                    struct_registry)
+from numba.targets import builtins, iterators, rangeobj
 
 
 LTYPEMAP = {
@@ -34,10 +35,6 @@ LTYPEMAP = {
 STRUCT_TYPES = {
     types.complex64: builtins.Complex64,
     types.complex128: builtins.Complex128,
-    types.range_state32_type: builtins.RangeState32,
-    types.range_iter32_type: builtins.RangeIter32,
-    types.range_state64_type: builtins.RangeState64,
-    types.range_iter64_type: builtins.RangeIter64,
     types.slice3_type: builtins.Slice,
 }
 
@@ -278,10 +275,6 @@ class BaseContext(object):
             dtys = [self.get_value_type(t) for t in ty]
             return Type.struct(dtys)
 
-        elif isinstance(ty, types.UniTupleIter):
-            stty = self.get_struct_type(self.make_unituple_iter(ty))
-            return stty
-
         elif isinstance(ty, types.Record):
             # Record are represented as byte array
             return Type.struct([Type.array(Type.int(8), ty.size)])
@@ -297,6 +290,18 @@ class BaseContext(object):
         elif ty in STRUCT_TYPES:
             return self.get_struct_type(STRUCT_TYPES[ty])
 
+        else:
+            try:
+                impl = struct_registry.match(ty)
+            except KeyError:
+                pass
+            else:
+                return self.get_struct_type(impl(ty))
+
+        if isinstance(ty, types.Pair):
+            pairty = self.make_pair(ty.first_type, ty.second_type)
+            return self.get_struct_type(pairty)
+
         else:
             return LTYPEMAP[ty]
 
@@ -348,6 +353,17 @@ class BaseContext(object):
     def is_struct_type(self, ty):
         return cgutils.is_struct(self.get_data_type(ty))
 
+    def get_constant_generic(self, builder, ty, val):
+        """
+        Return a LLVM constant representing value *val* of Numba type *ty*.
+        """
+        if self.is_struct_type(ty):
+            return self.get_constant_struct(builder, ty, val)
+        elif ty == types.string:
+            return self.get_constant_string(builder, ty, val)
+        else:
+            return self.get_constant(ty, val)
+
     def get_constant_struct(self, builder, ty, val):
         assert self.is_struct_type(ty)
         module = cgutils.get_module(builder)
@@ -363,12 +379,12 @@ class BaseContext(object):
             real = self.get_constant(innertype, val.real)
             imag = self.get_constant(innertype, val.imag)
             const = Constant.struct([real, imag])
+            return const
 
-            gv = module.add_global_variable(const.type, name=".const")
-            gv.linkage = lc.LINKAGE_INTERNAL
-            gv.initializer = const
-            gv.global_constant = True
-            return builder.load(gv)
+        elif isinstance(ty, types.Tuple):
+            consts = [self.get_constant_generic(builder, ty.types[i], v)
+                      for i, v in enumerate(val)]
+            return Constant.struct(consts)
 
         else:
             raise NotImplementedError(ty)
@@ -388,6 +404,9 @@ class BaseContext(object):
         elif ty in types.signed_domain:
             return Constant.int_signextend(lty, val)
 
+        elif ty in types.unsigned_domain:
+            return Constant.int(lty, val)
+
         elif ty in types.real_domain:
             return Constant.real(lty, val)
 
@@ -471,7 +490,7 @@ class BaseContext(object):
         """
         if ty == types.boolean:
             return builder.trunc(val, self.get_value_type(ty))
-        elif self.is_struct_type(ty):
+        elif cgutils.is_struct_ptr(val.type):
             return builder.load(val)
         return val
 
@@ -531,6 +550,22 @@ class BaseContext(object):
         assert code > 0
         builder.ret(Constant.int(Type.int(), code))
 
+    def pair_first(self, builder, val, ty):
+        """
+        Extract the first element of a heterogenous pair.
+        """
+        paircls = self.make_pair(ty.first_type, ty.second_type)
+        pair = paircls(self, builder, value=val)
+        return self.get_argument_value(builder, ty.first_type, pair.first)
+
+    def pair_second(self, builder, val, ty):
+        """
+        Extract the second element of a heterogenous pair.
+        """
+        paircls = self.make_pair(ty.first_type, ty.second_type)
+        pair = paircls(self, builder, value=val)
+        return self.get_argument_value(builder, ty.second_type, pair.second)
+
     def cast(self, builder, val, fromty, toty):
         if fromty == toty or toty == types.Any or isinstance(toty, types.Kind):
             return val
@@ -628,6 +663,16 @@ class BaseContext(object):
                 tup = builder.insert_value(tup, val, idx)
             return tup
 
+        elif (types.is_int_tuple(toty) and types.is_int_tuple(fromty) and
+                len(toty) == len(fromty)):
+            olditems = cgutils.unpack_tuple(builder, val, len(fromty))
+            items = [self.cast(builder, i, t, toty.dtype)
+                     for i, t in zip(olditems, fromty.types)]
+            tup = self.get_constant_undef(toty)
+            for idx, val in enumerate(items):
+                tup = builder.insert_value(tup, val, idx)
+            return tup
+
         elif toty == types.boolean:
             return self.is_true(builder, fromty, val)
 
@@ -747,8 +792,11 @@ class BaseContext(object):
         cls, _ = builtins.get_complex_info(typ)
         return cls
 
-    def make_unituple_iter(self, typ):
-        return builtins.make_unituple_iter(typ)
+    def make_pair(self, first_type, second_type):
+        """
+        Create a heterogenous pair class parametered for the given types.
+        """
+        return builtins.make_pair(first_type, second_type)
 
     def make_constant_array(self, builder, typ, ary):
         assert typ.layout == 'C'                # assumed in typeinfer.py
@@ -802,7 +850,16 @@ class BaseContext(object):
         n = len(self.exceptions) + errcode.ERROR_COUNT
         self.exceptions[n] = exc
         return n
-    
+
+    def optimize_function(self, func):
+        """
+        Perform function-level optimization.
+        This may improve generated code and reduce memory usage.
+
+        Note: This is called at the end of lowering.
+        """
+        pass
+
 
 class _wrap_impl(object):
     def __init__(self, imp, context, sig):
