@@ -1,11 +1,17 @@
 from __future__ import print_function
-import numba.unittest_support as unittest
+
+import itertools
+import operator
+import warnings
+
 import numpy as np
+
 from numba.compiler import compile_isolated, Flags
 from numba import types, typeinfer
 from numba.config import PYVERSION
+from .support import TestCase
 from numba.tests.true_div_usecase import truediv_usecase
-import itertools
+import numba.unittest_support as unittest
 
 Noflags = Flags()
 
@@ -13,76 +19,369 @@ enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
 
 
-def add_usecase(x, y):
-    return x + y
+class LiteralOperatorImpl(object):
 
-def sub_usecase(x, y):
-    return x - y
+    @staticmethod
+    def add_usecase(x, y):
+        return x + y
 
-def mul_usecase(x, y):
-    return x * y
+    @staticmethod
+    def sub_usecase(x, y):
+        return x - y
 
-def div_usecase(x, y):
-    return x / y
+    @staticmethod
+    def mul_usecase(x, y):
+        return x * y
 
-def floordiv_usecase(x, y):
-    return x / y
+    @staticmethod
+    def div_usecase(x, y):
+        return x / y
 
-def mod_usecase(x, y):
-    return x % y
+    @staticmethod
+    def floordiv_usecase(x, y):
+        return x // y
 
-def pow_usecase(x, y):
-    return x ** y
+    truediv_usecase = staticmethod(truediv_usecase)
 
-def bitshift_left_usecase(x, y):
-    return x << y
+    @staticmethod
+    def mod_usecase(x, y):
+        return x % y
 
-def bitshift_right_usecase(x, y):
-    return x >> y
+    @staticmethod
+    def pow_usecase(x, y):
+        return x ** y
 
-def bitwise_and_usecase(x, y):
-    return x & y
+    @staticmethod
+    def bitshift_left_usecase(x, y):
+        return x << y
 
-def bitwise_or_usecase(x, y):
-    return x | y
+    @staticmethod
+    def bitshift_right_usecase(x, y):
+        return x >> y
 
-def bitwise_xor_usecase(x, y):
-    return x ^ y
+    @staticmethod
+    def bitwise_and_usecase(x, y):
+        return x & y
 
-def bitwise_not_usecase(x, y):
-    return ~x
+    @staticmethod
+    def bitwise_or_usecase(x, y):
+        return x | y
 
-def not_usecase(x):
-    return not(x)
+    @staticmethod
+    def bitwise_xor_usecase(x, y):
+        return x ^ y
 
-def negate_usecase(x):
-    return -x
+    @staticmethod
+    def bitwise_not_usecase(x, y):
+        return ~x
 
-def unary_positive_usecase(x):
-    return +x
+    @staticmethod
+    def not_usecase(x):
+        return not(x)
 
-class TestOperators(unittest.TestCase):
+    @staticmethod
+    def negate_usecase(x):
+        return -x
+
+    @staticmethod
+    def unary_positive_usecase(x):
+        return +x
+
+    @staticmethod
+    def lt_usecase(x, y):
+        return x < y
+
+    @staticmethod
+    def le_usecase(x, y):
+        return x <= y
+
+    @staticmethod
+    def gt_usecase(x, y):
+        return x > y
+
+    @staticmethod
+    def ge_usecase(x, y):
+        return x >= y
+
+    @staticmethod
+    def eq_usecase(x, y):
+        return x == y
+
+    @staticmethod
+    def ne_usecase(x, y):
+        return x != y
+
+
+class FunctionalOperatorImpl(object):
+
+    @staticmethod
+    def add_usecase(x, y):
+        return operator.add(x, y)
+
+    @staticmethod
+    def sub_usecase(x, y):
+        return operator.sub(x, y)
+
+    @staticmethod
+    def mul_usecase(x, y):
+        return operator.mul(x, y)
+
+    if PYVERSION >= (3, 0):
+        div_usecase = NotImplemented
+    else:
+        @staticmethod
+        def div_usecase(x, y):
+            return operator.div(x, y)
+
+    @staticmethod
+    def floordiv_usecase(x, y):
+        return operator.floordiv(x, y)
+
+    @staticmethod
+    def truediv_usecase(x, y):
+        return operator.truediv(x, y)
+
+    @staticmethod
+    def mod_usecase(x, y):
+        return operator.mod(x, y)
+
+    @staticmethod
+    def pow_usecase(x, y):
+        return operator.pow(x, y)
+
+    @staticmethod
+    def bitshift_left_usecase(x, y):
+        return operator.lshift(x, y)
+
+    @staticmethod
+    def bitshift_right_usecase(x, y):
+        return operator.rshift(x, y)
+
+    @staticmethod
+    def bitwise_and_usecase(x, y):
+        return operator.and_(x, y)
+
+    @staticmethod
+    def bitwise_or_usecase(x, y):
+        return operator.or_(x, y)
+
+    @staticmethod
+    def bitwise_xor_usecase(x, y):
+        return operator.xor(x, y)
+
+    @staticmethod
+    def bitwise_not_usecase(x, y):
+        return operator.invert(x)
+
+    @staticmethod
+    def not_usecase(x):
+        return operator.not_(x)
+
+    @staticmethod
+    def negate_usecase(x):
+        return operator.neg(x)
+
+    @staticmethod
+    def unary_positive_usecase(x):
+        return operator.pos(x)
+
+    @staticmethod
+    def lt_usecase(x, y):
+        return operator.lt(x, y)
+
+    @staticmethod
+    def le_usecase(x, y):
+        return operator.le(x, y)
+
+    @staticmethod
+    def gt_usecase(x, y):
+        return operator.gt(x, y)
+
+    @staticmethod
+    def ge_usecase(x, y):
+        return operator.ge(x, y)
+
+    @staticmethod
+    def eq_usecase(x, y):
+        return operator.eq(x, y)
+
+    @staticmethod
+    def ne_usecase(x, y):
+        return operator.ne(x, y)
+
+
+class TestOperators(TestCase):
+
+    op = LiteralOperatorImpl
 
     def run_test_ints(self, pyfunc, x_operands, y_operands, types_list,
                       flags=enable_pyobj_flags):
+        if pyfunc is NotImplemented:
+            self.skipTest("test irrelevant on this version of Python")
         for arg_types in types_list:
             cr = compile_isolated(pyfunc, arg_types, flags=flags)
             cfunc = cr.entry_point
 
             for x, y in itertools.product(x_operands, y_operands):
-                self.assertTrue(np.all(pyfunc(x, y) == cfunc(x, y)))
+                got = cfunc(x, y)
+                expected = pyfunc(x, y)
+                self.assertTrue(np.all(got == expected),
+                                "mismatch: %r != %r" % (got, expected))
 
     def run_test_floats(self, pyfunc, x_operands, y_operands, types_list,
                         flags=enable_pyobj_flags):
+        if pyfunc is NotImplemented:
+            self.skipTest("test irrelevant on this version of Python")
         for arg_types in types_list:
             cr = compile_isolated(pyfunc, arg_types, flags=flags)
             cfunc = cr.entry_point
             for x, y in itertools.product(x_operands, y_operands):
                 self.assertTrue(np.allclose(pyfunc(x, y), cfunc(x, y)))
 
+    def coerce_operand(self, op, numba_type):
+        if hasattr(op, "dtype"):
+            return numba_type.cast_python_value(op)
+        elif numba_type in types.unsigned_domain:
+            return abs(int(op.real))
+        elif numba_type in types.integer_domain:
+            return int(op.real)
+        elif numba_type in types.real_domain:
+            return float(op.real)
+        else:
+            return op
+
+    def run_test_scalar_compare(self, pyfunc, flags=enable_pyobj_flags,
+                                ordered=True):
+        ops = self.compare_scalar_operands
+        types_list = self.compare_types
+        if not ordered:
+            types_list = types_list + self.compare_unordered_types
+        for typ in types_list:
+            cr = compile_isolated(pyfunc, (typ, typ), flags=flags)
+            cfunc = cr.entry_point
+            for x, y in itertools.product(ops, ops):
+                x = self.coerce_operand(x, typ)
+                y = self.coerce_operand(y, typ)
+                expected = pyfunc(x, y)
+                got = cfunc(x, y)
+                # Scalar ops => scalar result
+                self.assertIs(type(got), type(expected))
+                self.assertEqual(got, expected,
+                                 "mismatch with %r (%r, %r)"
+                                 % (typ, x, y))
+
+    def run_test_array_compare(self, pyfunc, flags=enable_pyobj_flags,
+                               ordered=True):
+        ops = np.array(self.compare_scalar_operands)
+        types_list = self.compare_types
+        if not ordered:
+            types_list = types_list + self.compare_unordered_types
+        for typ in types_list:
+            array_type = types.Array(typ, 1, 'C')
+            cr = compile_isolated(pyfunc, (array_type, array_type),
+                                  flags=flags)
+            cfunc = cr.entry_point
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore', np.ComplexWarning)
+                arr = typ.cast_python_value(ops)
+            for i in range(len(arr)):
+                x = arr
+                y = np.concatenate((arr[i:], arr[:i]))
+                expected = pyfunc(x, y)
+                got = cfunc(x, y)
+                # Array ops => array result
+                self.assertEqual(got.dtype, expected.dtype)
+                self.assertTrue(np.all(got == expected),
+                                "mismatch with %r (%r, %r): %r != %r"
+                                % (typ, x, y, got, expected))
+
+    compare_scalar_operands = [-0.5, -1.0 + 1j, -1.0 + 2j, -0.5 + 1j, 1.5]
+    compare_types = [types.int32, types.int64,
+                     types.uint32, types.uint64,
+                     types.float32, types.float64]
+    compare_unordered_types = [types.complex64, types.complex128]
+
+    def test_lt_scalar(self, flags=enable_pyobj_flags):
+        self.run_test_scalar_compare(self.op.lt_usecase, flags)
+
+    def test_lt_scalar_npm(self):
+        self.test_lt_scalar(flags=Noflags)
+
+    def test_le_scalar(self, flags=enable_pyobj_flags):
+        self.run_test_scalar_compare(self.op.le_usecase, flags)
+
+    def test_le_scalar_npm(self):
+        self.test_le_scalar(flags=Noflags)
+
+    def test_gt_scalar(self, flags=enable_pyobj_flags):
+        self.run_test_scalar_compare(self.op.gt_usecase, flags)
+
+    def test_gt_scalar_npm(self):
+        self.test_gt_scalar(flags=Noflags)
+
+    def test_ge_scalar(self, flags=enable_pyobj_flags):
+        self.run_test_scalar_compare(self.op.ge_usecase, flags)
+
+    def test_ge_scalar_npm(self):
+        self.test_ge_scalar(flags=Noflags)
+
+    def test_eq_scalar(self, flags=enable_pyobj_flags):
+        self.run_test_scalar_compare(self.op.eq_usecase, flags, ordered=False)
+
+    def test_eq_scalar_npm(self):
+        self.test_eq_scalar(flags=Noflags)
+
+    def test_ne_scalar(self, flags=enable_pyobj_flags):
+        self.run_test_scalar_compare(self.op.ne_usecase, flags, ordered=False)
+
+    def test_ne_scalar_npm(self):
+        self.test_ne_scalar(flags=Noflags)
+
+    def test_eq_array(self, flags=enable_pyobj_flags):
+        self.run_test_array_compare(self.op.eq_usecase, flags, ordered=False)
+
+    def test_eq_array_npm(self):
+        with self.assertTypingError():
+            self.test_eq_array(flags=Noflags)
+
+    def test_ne_array(self, flags=enable_pyobj_flags):
+        self.run_test_array_compare(self.op.ne_usecase, flags, ordered=False)
+
+    def test_ne_array_npm(self):
+        with self.assertTypingError():
+            self.test_ne_array(flags=Noflags)
+
+    def test_lt_array(self, flags=enable_pyobj_flags):
+        self.run_test_array_compare(self.op.lt_usecase, flags)
+
+    def test_lt_array_npm(self):
+        with self.assertTypingError():
+            self.test_lt_array(flags=Noflags)
+
+    def test_le_array(self, flags=enable_pyobj_flags):
+        self.run_test_array_compare(self.op.le_usecase, flags)
+
+    def test_le_array_npm(self):
+        with self.assertTypingError():
+            self.test_le_array(flags=Noflags)
+
+    def test_gt_array(self, flags=enable_pyobj_flags):
+        self.run_test_array_compare(self.op.gt_usecase, flags)
+
+    def test_gt_array_npm(self):
+        with self.assertTypingError():
+            self.test_gt_array(flags=Noflags)
+
+    def test_ge_array(self, flags=enable_pyobj_flags):
+        self.run_test_array_compare(self.op.ge_usecase, flags)
+
+    def test_ge_array_npm(self):
+        with self.assertTypingError():
+            self.test_ge_array(flags=Noflags)
+
     def test_add_ints(self, flags=enable_pyobj_flags):
 
-        pyfunc = add_usecase
+        pyfunc = self.op.add_usecase
 
         x_operands = [-1, 0, 1]
         y_operands = [-1, 0, 1]
@@ -105,7 +404,7 @@ class TestOperators(unittest.TestCase):
 
     def test_add_ints_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = add_usecase
+        pyfunc = self.op.add_usecase
 
         array = np.arange(-10, 10, dtype=np.int32)
 
@@ -121,13 +420,13 @@ class TestOperators(unittest.TestCase):
     def test_add_ints_npm(self):
         self.test_add_ints(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_add_ints_array_npm(self):
-        self.test_add_ints_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_add_ints_array(flags=Noflags)
 
     def test_add_floats(self, flags=enable_pyobj_flags):
 
-        pyfunc = add_usecase
+        pyfunc = self.op.add_usecase
 
         x_operands = [-1.1, 0.0, 1.1]
         y_operands = [-1.1, 0.0, 1.1]
@@ -139,7 +438,7 @@ class TestOperators(unittest.TestCase):
                              flags=flags)
 
     def test_add_floats_array(self, flags=enable_pyobj_flags):
-        pyfunc = add_usecase
+        pyfunc = self.op.add_usecase
         array = np.arange(-1, 1, 0.1, dtype=np.float32)
 
         x_operands = [array]
@@ -155,13 +454,13 @@ class TestOperators(unittest.TestCase):
         self.test_add_floats(flags=Noflags)
 
 
-    @unittest.expectedFailure
     def test_add_floats_array_npm(self):
-        self.test_add_floats_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_add_floats_array(flags=Noflags)
 
     def test_sub_ints(self, flags=enable_pyobj_flags):
 
-        pyfunc = sub_usecase
+        pyfunc = self.op.sub_usecase
 
         x_operands = [-1, 0, 1]
         y_operands = [-1, 0, 1]
@@ -185,7 +484,7 @@ class TestOperators(unittest.TestCase):
 
     def test_sub_ints_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = sub_usecase
+        pyfunc = self.op.sub_usecase
 
         array = np.arange(-10, 10, dtype=np.int32)
 
@@ -201,13 +500,13 @@ class TestOperators(unittest.TestCase):
     def test_sub_ints_npm(self):
         self.test_sub_ints(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_sub_ints_array_npm(self):
-        self.test_sub_ints_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_sub_ints_array(flags=Noflags)
 
     def test_sub_floats(self, flags=enable_pyobj_flags):
 
-        pyfunc = sub_usecase
+        pyfunc = self.op.sub_usecase
 
         x_operands = [-1.1, 0.0, 1.1]
         y_operands = [-1.1, 0.0, 1.1]
@@ -220,7 +519,7 @@ class TestOperators(unittest.TestCase):
 
     def test_sub_floats_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = sub_usecase
+        pyfunc = self.op.sub_usecase
 
         array = np.arange(-1, 1, 0.1, dtype=np.float32)
 
@@ -236,13 +535,13 @@ class TestOperators(unittest.TestCase):
     def test_sub_floats_npm(self):
         self.test_sub_floats(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_sub_floats_array_npm(self):
-        self.test_sub_floats_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_sub_floats_array(flags=Noflags)
 
     def test_mul_ints(self, flags=enable_pyobj_flags):
 
-        pyfunc = mul_usecase
+        pyfunc = self.op.mul_usecase
 
         x_operands = [-1, 0, 1]
         y_operands = [-1, 0, 1]
@@ -265,7 +564,7 @@ class TestOperators(unittest.TestCase):
 
     def test_mul_ints_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = mul_usecase
+        pyfunc = self.op.mul_usecase
 
         array = np.arange(-10, 10, dtype=np.int32)
 
@@ -281,13 +580,13 @@ class TestOperators(unittest.TestCase):
     def test_mul_ints_npm(self):
         self.test_mul_ints(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_mul_ints_array_npm(self):
-        self.test_mul_ints_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_mul_ints_array(flags=Noflags)
 
     def test_mul_floats(self, flags=enable_pyobj_flags):
 
-        pyfunc = mul_usecase
+        pyfunc = self.op.mul_usecase
 
         x_operands = [-111.111, 0.0, 111.111]
         y_operands = [-111.111, 0.0, 111.111]
@@ -300,7 +599,7 @@ class TestOperators(unittest.TestCase):
 
     def test_mul_floats_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = mul_usecase
+        pyfunc = self.op.mul_usecase
         array = np.arange(-1, 1, 0.1, dtype=np.float32)
 
         x_operands = [array]
@@ -315,9 +614,9 @@ class TestOperators(unittest.TestCase):
     def test_mul_floats_npm(self):
         self.test_mul_floats(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_mul_floats_array_npm(self):
-        self.test_mul_floats_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_mul_floats_array(flags=Noflags)
 
     def test_div_ints(self, flags=enable_pyobj_flags):
         if PYVERSION >= (3, 0):
@@ -326,7 +625,7 @@ class TestOperators(unittest.TestCase):
         else:
             tester = self.run_test_ints
 
-        pyfunc = div_usecase
+        pyfunc = self.op.div_usecase
 
         x_operands = [-1, 0, 1, 2, 3]
         y_operands = [-3, -2, -1, 1]
@@ -346,7 +645,7 @@ class TestOperators(unittest.TestCase):
         tester(pyfunc, x_operands, y_operands, types_list, flags=flags)
 
     def test_div_ints_array(self, flags=enable_pyobj_flags):
-        pyfunc = div_usecase
+        pyfunc = self.op.div_usecase
         array = np.array([-10, -9, -2, -1, 1, 2, 9, 10], dtype=np.int32)
 
         x_operands = [array]
@@ -361,13 +660,13 @@ class TestOperators(unittest.TestCase):
     def test_div_ints_npm(self):
         self.test_div_ints(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_div_ints_array_npm(self):
-        self.test_div_ints_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_div_ints_array(flags=Noflags)
 
     def test_div_floats(self, flags=enable_pyobj_flags):
 
-        pyfunc = div_usecase
+        pyfunc = self.op.div_usecase
 
         x_operands = [-111.111, 0.0, 2.2]
         y_operands = [-2.2, 1.0, 111.111]
@@ -380,7 +679,7 @@ class TestOperators(unittest.TestCase):
 
     def test_div_floats_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = div_usecase
+        pyfunc = self.op.div_usecase
 
         array = np.concatenate((np.arange(0.1, 1.1, 0.1, dtype=np.float32),
                                np.arange(-1.0, 0.0, 0.1, dtype=np.float32)))
@@ -397,12 +696,12 @@ class TestOperators(unittest.TestCase):
     def test_div_floats_npm(self):
         self.test_div_floats(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_div_floats_array_npm(self):
-        self.test_div_floats_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_div_floats_array(flags=Noflags)
 
     def test_truediv_ints(self, flags=enable_pyobj_flags):
-        pyfunc = truediv_usecase
+        pyfunc = self.op.truediv_usecase
 
         x_operands = [0, 1, 2, 3]
         y_operands = [1, 1, 2, 3]
@@ -419,7 +718,7 @@ class TestOperators(unittest.TestCase):
         self.test_truediv_ints(flags=Noflags)
 
     def test_truediv_floats(self, flags=enable_pyobj_flags):
-        pyfunc = truediv_usecase
+        pyfunc = self.op.truediv_usecase
 
         x_operands = [-111.111, 0.0, 2.2]
         y_operands = [-2.2, 1.0, 111.111]
@@ -434,7 +733,7 @@ class TestOperators(unittest.TestCase):
         self.test_truediv_floats(flags=Noflags)
 
     def test_floordiv_floats(self, flags=enable_pyobj_flags):
-        pyfunc = floordiv_usecase
+        pyfunc = self.op.floordiv_usecase
 
         x_operands = [-111.111, 0.0, 2.2]
         y_operands = [-2.2, 1.0, 111.111]
@@ -450,7 +749,7 @@ class TestOperators(unittest.TestCase):
 
     def test_mod_ints(self, flags=enable_pyobj_flags):
 
-        pyfunc = mod_usecase
+        pyfunc = self.op.mod_usecase
 
         x_operands = [-1, 0, 1, 2, 3]
         y_operands = [-3, -2, -1, 1]
@@ -473,7 +772,7 @@ class TestOperators(unittest.TestCase):
 
     def test_mod_ints_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = mod_usecase
+        pyfunc = self.op.mod_usecase
 
         array = np.concatenate((np.arange(1, 11, dtype=np.int32),
                                np.arange(-10, 0, dtype=np.int32)))
@@ -490,13 +789,13 @@ class TestOperators(unittest.TestCase):
     def test_mod_ints_npm(self):
         self.test_mod_ints(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_mod_ints_array_npm(self):
-        self.test_mod_ints_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_mod_ints_array(flags=Noflags)
 
     def test_mod_floats(self, flags=enable_pyobj_flags):
 
-        pyfunc = mod_usecase
+        pyfunc = self.op.mod_usecase
 
         x_operands = [-111.111, 0.0, 2.2]
         y_operands = [-2.2, 1.0, 111.111]
@@ -509,7 +808,7 @@ class TestOperators(unittest.TestCase):
 
     def test_mod_floats_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = mod_usecase
+        pyfunc = self.op.mod_usecase
 
         array = np.arange(-1, 1, 0.1, dtype=np.float32)
 
@@ -525,13 +824,13 @@ class TestOperators(unittest.TestCase):
     def test_mod_floats_npm(self):
         self.test_mod_floats(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_mod_floats_array_npm(self):
-        self.test_mod_floats_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_mod_floats_array(flags=Noflags)
 
     def test_pow_ints(self, flags=enable_pyobj_flags):
 
-        pyfunc = pow_usecase
+        pyfunc = self.op.pow_usecase
 
         x_operands = [-2, -1, 0, 1, 2]
         y_operands = [0, 1, 2]
@@ -554,7 +853,7 @@ class TestOperators(unittest.TestCase):
 
     def test_pow_ints_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = pow_usecase
+        pyfunc = self.op.pow_usecase
 
         array = np.arange(-10, 10, dtype=np.int32)
 
@@ -570,13 +869,13 @@ class TestOperators(unittest.TestCase):
     def test_pow_ints_npm(self):
         self.test_pow_ints(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_pow_ints_array_npm(self):
-        self.test_pow_ints_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_pow_ints_array(flags=Noflags)
 
     def test_pow_floats(self, flags=enable_pyobj_flags):
 
-        pyfunc = pow_usecase
+        pyfunc = self.op.pow_usecase
 
         x_operands = [-222.222, -111.111, 111.111, 222.222]
         y_operands = [-2, -1, 0, 1, 2]
@@ -598,7 +897,7 @@ class TestOperators(unittest.TestCase):
 
     def test_pow_floats_array(self, flags=enable_pyobj_flags):
 
-        pyfunc = pow_usecase
+        pyfunc = self.op.pow_usecase
         # NOTE
         # If x is finite negative and y is finite but not an integer,
         # it causes a domain error
@@ -629,12 +928,12 @@ class TestOperators(unittest.TestCase):
     def test_pow_floats_npm(self):
         self.test_pow_floats(flags=Noflags)
 
-    @unittest.expectedFailure
     def test_pow_floats_array_npm(self):
-        self.test_pow_floats_array(flags=Noflags)
+        with self.assertTypingError():
+            self.test_pow_floats_array(flags=Noflags)
 
     def test_add_complex(self, flags=enable_pyobj_flags):
-        pyfunc = add_usecase
+        pyfunc = self.op.add_usecase
 
         x_operands = [1+0j, 1j, -1-1j]
         y_operands = x_operands
@@ -649,7 +948,7 @@ class TestOperators(unittest.TestCase):
         self.test_add_complex(flags=Noflags)
 
     def test_sub_complex(self, flags=enable_pyobj_flags):
-        pyfunc = sub_usecase
+        pyfunc = self.op.sub_usecase
 
         x_operands = [1+0j, 1j, -1-1j]
         y_operands = [1, 2, 3]
@@ -664,7 +963,7 @@ class TestOperators(unittest.TestCase):
         self.test_sub_complex(flags=Noflags)
 
     def test_mul_complex(self, flags=enable_pyobj_flags):
-        pyfunc = mul_usecase
+        pyfunc = self.op.mul_usecase
 
         x_operands = [1+0j, 1j, -1-1j]
         y_operands = [1, 2, 3]
@@ -679,7 +978,7 @@ class TestOperators(unittest.TestCase):
         self.test_mul_complex(flags=Noflags)
 
     def test_div_complex(self, flags=enable_pyobj_flags):
-        pyfunc = div_usecase
+        pyfunc = self.op.div_usecase
 
         x_operands = [1+0j, 1j, -1-1j]
         y_operands = [1, 2, 3]
@@ -694,7 +993,7 @@ class TestOperators(unittest.TestCase):
         self.test_div_complex(flags=Noflags)
 
     def test_truediv_complex(self, flags=enable_pyobj_flags):
-        pyfunc = truediv_usecase
+        pyfunc = self.op.truediv_usecase
 
         x_operands = [1+0j, 1j, -1-1j]
         y_operands = [1, 2, 3]
@@ -709,21 +1008,16 @@ class TestOperators(unittest.TestCase):
         self.test_truediv_complex(flags=Noflags)
 
     def test_mod_complex(self, flags=enable_pyobj_flags):
-        pyfunc = mod_usecase
-
-        try:
+        pyfunc = self.op.mod_usecase
+        with self.assertTypingError():
             cres = compile_isolated(pyfunc, (types.complex64, types.complex64))
-        except typeinfer.TypingError as e:
-            e.msg.startswith("Undeclared %(complex64, complex64)")
-        else:
-            self.fail("Complex % should trigger an undeclared error")
 
     def test_mod_complex_npm(self):
         self.test_mod_complex(flags=Noflags)
 
     def test_bitshift_left(self, flags=enable_pyobj_flags):
 
-        pyfunc = bitshift_left_usecase
+        pyfunc = self.op.bitshift_left_usecase
 
         x_operands = [0, 1]
         y_operands = [0, 1, 2, 4, 8, 16, 31]
@@ -762,7 +1056,7 @@ class TestOperators(unittest.TestCase):
 
     def test_bitshift_right(self, flags=enable_pyobj_flags):
 
-        pyfunc = bitshift_right_usecase
+        pyfunc = self.op.bitshift_right_usecase
 
         x_operands = [0, 1, 2**32 - 1]
         y_operands = [0, 1, 2, 4, 8, 16, 31]
@@ -801,7 +1095,7 @@ class TestOperators(unittest.TestCase):
 
     def test_bitwise_and(self, flags=enable_pyobj_flags):
 
-        pyfunc = bitwise_and_usecase
+        pyfunc = self.op.bitwise_and_usecase
 
         x_operands = list(range(0, 8)) + [2**32 - 1]
         y_operands = list(range(0, 8)) + [2**32 - 1]
@@ -840,7 +1134,7 @@ class TestOperators(unittest.TestCase):
 
     def test_bitwise_or(self, flags=enable_pyobj_flags):
 
-        pyfunc = bitwise_or_usecase
+        pyfunc = self.op.bitwise_or_usecase
 
         x_operands = list(range(0, 8)) + [2**32 - 1]
         y_operands = list(range(0, 8)) + [2**32 - 1]
@@ -879,7 +1173,7 @@ class TestOperators(unittest.TestCase):
 
     def test_bitwise_xor(self, flags=enable_pyobj_flags):
 
-        pyfunc = bitwise_xor_usecase
+        pyfunc = self.op.bitwise_xor_usecase
 
         x_operands = list(range(0, 8)) + [2**32 - 1]
         y_operands = list(range(0, 8)) + [2**32 - 1]
@@ -918,7 +1212,7 @@ class TestOperators(unittest.TestCase):
 
     def test_bitwise_not(self, flags=enable_pyobj_flags):
 
-        pyfunc = bitwise_not_usecase
+        pyfunc = self.op.bitwise_not_usecase
 
         x_operands = list(range(0, 8)) + [2**32 - 1]
         x_operands = [np.uint32(x) for x in x_operands]
@@ -958,7 +1252,7 @@ class TestOperators(unittest.TestCase):
         self.test_bitwise_not(flags=Noflags)
 
     def test_not(self):
-        pyfunc = not_usecase
+        pyfunc = self.op.not_usecase
 
         values = [
             1,
@@ -974,7 +1268,7 @@ class TestOperators(unittest.TestCase):
             self.assertEqual(pyfunc(val), cfunc(val))
 
     def test_not_npm(self):
-        pyfunc = not_usecase
+        pyfunc = self.op.not_usecase
         # test native mode
         argtys = [
             types.int8,
@@ -997,7 +1291,7 @@ class TestOperators(unittest.TestCase):
             self.assertEqual(pyfunc(val), cfunc(val))
 
     def test_negate_npm(self):
-        pyfunc = negate_usecase
+        pyfunc = self.op.negate_usecase
         # test native mode
         argtys = [
             types.int8,
@@ -1021,7 +1315,7 @@ class TestOperators(unittest.TestCase):
             self.assertAlmostEqual(pyfunc(val), cfunc(val))
 
     def test_negate(self):
-        pyfunc = negate_usecase
+        pyfunc = self.op.negate_usecase
         values = [
             1,
             2,
@@ -1035,7 +1329,7 @@ class TestOperators(unittest.TestCase):
             self.assertEqual(pyfunc(val), cfunc(val))
 
     def test_unary_positive_npm(self):
-        pyfunc = unary_positive_usecase
+        pyfunc = self.op.unary_positive_usecase
         # test native mode
         argtys = [
             types.int8,
@@ -1059,7 +1353,7 @@ class TestOperators(unittest.TestCase):
             self.assertAlmostEqual(pyfunc(val), cfunc(val))
 
     def test_unary_positive(self):
-        pyfunc = unary_positive_usecase
+        pyfunc = self.op.unary_positive_usecase
         values = [
             1,
             2,
@@ -1073,6 +1367,11 @@ class TestOperators(unittest.TestCase):
             self.assertEqual(pyfunc(val), cfunc(val))
 
 
+class TestOperatorModule(TestOperators):
+
+    op = FunctionalOperatorImpl
+
+
 if __name__ == '__main__':
     unittest.main()
 
