@@ -1,12 +1,20 @@
 from __future__ import print_function
+
+import itertools
 import sys
 import warnings
-import numba.unittest_support as unittest
+
 import numpy as np
-from numba.compiler import compile_isolated, Flags
+import functools
+
+import numba.unittest_support as unittest
+from numba.compiler import compile_isolated, Flags, DEFAULT_FLAGS
 from numba import types, utils
 from numba.config import PYVERSION
-import itertools
+from numba.targets import cpu
+from numba.typeinfer import TypingError
+from numba.tests.support import TestCase, CompilationCache
+
 
 is32bits = tuple.__itemsize__ == 4
 iswindows = sys.platform.startswith('win32')
@@ -16,6 +24,17 @@ enable_pyobj_flags.set("enable_pyobject")
 
 no_pyobj_flags = Flags()
 
+def _unimplemented(func):
+    """An 'expectedFailure' like decorator that only expects compilation errors
+    caused by unimplemented functions that fail in no-python mode"""
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        try:
+            func(*args, **kwargs)
+        except TypingError:
+            raise unittest._ExpectedFailure(sys.exc_info())
+        raise unittest._UnexpectedSuccess
+
 def _make_unary_ufunc_usecase(ufunc_name):
     ldict = {}
     exec("def fn(x,out):\n    np.{0}(x,out)".format(ufunc_name), globals(), ldict)
@@ -32,7 +51,7 @@ def _make_binary_ufunc_usecase(ufunc_name):
     return fn
 
 
-class TestUFuncs(unittest.TestCase):
+class TestUFuncs(TestCase):
 
     def setUp(self):
         self.inputs = [
@@ -60,7 +79,9 @@ class TestUFuncs(unittest.TestCase):
             (np.array([-1,0,1], dtype='i4'), types.Array(types.int32, 1, 'C')),
             (np.array([-1,0,1], dtype='i8'), types.Array(types.int64, 1, 'C')),
             (np.array([-0.5, 0.0, 0.5], dtype='f4'), types.Array(types.float32, 1, 'C')),
-            (np.array([-0.5, 0.0, 0.5], dtype='f8'), types.Array(types.float64, 1, 'C'))]
+            (np.array([-0.5, 0.0, 0.5], dtype='f8'), types.Array(types.float64, 1, 'C')),
+            ]
+        self.cache = CompilationCache()
 
     def unary_ufunc_test(self, ufunc_name, flags=enable_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
@@ -106,7 +127,8 @@ class TestUFuncs(unittest.TestCase):
             if iswindows and output_type.dtype is types.uint64:
                 continue
 
-            cr = compile_isolated(pyfunc, (input_type, output_type), flags=flags)
+            cr = self.cache.compile(pyfunc, (input_type, output_type),
+                                    flags=flags)
             cfunc = cr.entry_point
 
             if isinstance(input_operand, np.ndarray):
@@ -195,8 +217,8 @@ class TestUFuncs(unittest.TestCase):
             if iswindows and output_type.dtype is types.uint64:
                 continue
 
-            cr = compile_isolated(pyfunc, (input_type, input_type, output_type),
-                                  flags=flags)
+            cr = self.cache.compile(pyfunc, (input_type, input_type, output_type),
+                                    flags=flags)
             cfunc = cr.entry_point
 
             if isinstance(input_operand, np.ndarray):
@@ -221,13 +243,78 @@ class TestUFuncs(unittest.TestCase):
                                 np.allclose(result, expected))
 
 
+    def unary_int_ufunc_test(self, name=None, flags=enable_pyobj_flags):
+        self.unary_ufunc_test(name, flags=flags,
+            skip_inputs=[types.float32, types.float64,
+                types.Array(types.float32, 1, 'C'),
+                types.Array(types.float64, 1, 'C')])
+
     def binary_int_ufunc_test(self, name=None, flags=enable_pyobj_flags):
         self.binary_ufunc_test(name, flags=flags,
             skip_inputs=[types.float32, types.float64,
                 types.Array(types.float32, 1, 'C'),
                 types.Array(types.float64, 1, 'C')])
 
-    # unary ufunc tests
+
+    ############################################################################
+    # Math operations
+    def test_add_ufunc(self, flags=enable_pyobj_flags):
+        self.binary_ufunc_test('add', flags=flags)
+
+    def test_add_ufunc_npm(self):
+        self.test_add_ufunc(flags=no_pyobj_flags)
+
+    def test_subtract_ufunc(self, flags=enable_pyobj_flags):
+        self.binary_ufunc_test('subtract', flags=flags)
+
+    def test_subtract_ufunc_npm(self):
+        self.test_subtract_ufunc(flags=no_pyobj_flags)
+
+    def test_multiply_ufunc(self, flags=enable_pyobj_flags):
+        self.binary_ufunc_test('multiply', flags=flags)
+
+    def test_multiply_ufunc_npm(self):
+        self.test_multiply_ufunc(flags=no_pyobj_flags)
+
+    def test_divide_ufunc(self, flags=enable_pyobj_flags):
+        # Bear in mind that in python3 divide IS true_divide
+        # so the out type for int types will be a double
+        int_out_type = None
+        if PYVERSION >= (3, 0):
+            int_out_type = types.float64
+
+        self.binary_ufunc_test('divide', flags=flags, int_output_type=int_out_type)
+
+    def test_divide_ufunc_npm(self):
+        self.test_divide_ufunc(flags=no_pyobj_flags)
+
+    def test_logaddexp_ufunc(self):
+        self.binary_ufunc_test('logaddexp')
+
+    @_unimplemented
+    def test_logaddexp_ufunc_npm(self):
+        self.binary_ufunc_test('logaddexp', flags=no_pyobj_flags)
+
+    def test_logaddexp2_ufunc(self):
+        self.binary_ufunc_test('logaddexp2')
+
+    @_unimplemented
+    def test_logaddexp2_ufunc_npm(self):
+        self.binary_ufunc_test('logaddexp2', flags=no_pyobj_flags)
+
+    def test_true_divide_ufunc(self, flags=enable_pyobj_flags):
+        self.binary_ufunc_test('true_divide', flags=flags, int_output_type=types.float64)
+
+    def test_true_divide_ufunc_npm(self):
+        self.test_true_divide_ufunc(flags=no_pyobj_flags)
+
+    def test_floor_divide_ufunc(self):
+        self.binary_ufunc_test('floor_divide')
+
+    @_unimplemented
+    def test_floor_divide_ufunc_npm(self):
+        self.binary_ufunc_test('floor_divide', flags=no_pyobj_flags)
+
     def test_negative_ufunc(self, flags=enable_pyobj_flags):
         # NumPy ufunc has bug with uint32 as input and int64 as output,
         # so skip uint32 input.
@@ -237,6 +324,45 @@ class TestUFuncs(unittest.TestCase):
     def test_negative_ufunc_npm(self):
         self.test_negative_ufunc(flags=no_pyobj_flags)
 
+    def test_power_ufunc(self):
+        self.binary_ufunc_test('power')
+
+    def test_power_ufunc_npm(self):
+        self.binary_ufunc_test('power', flags=no_pyobj_flags)
+
+    def test_remainder_ufunc(self):
+        self.binary_ufunc_test('remainder')
+
+    @_unimplemented
+    def test_remainder_ufunc_npm(self):
+        self.binary_ufunc_test('remainder', flags=no_pyobj_flags)
+
+    def test_mod_ufunc(self):
+        self.binary_ufunc_test('mod')
+
+    @_unimplemented
+    def test_mod_ufunc_npm(self):
+        self.binary_ufunc_test('mod', flags=no_pyobj_flags)
+
+    def test_fmod_ufunc(self):
+        self.binary_ufunc_test('fmod')
+
+    @_unimplemented
+    def test_fmod_ufunc_npm(self):
+        self.binary_ufunc_test('fmod', flags=no_pyobj_flags)
+
+    def test_abs_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('abs', flags=flags,
+            additional_inputs = [(np.iinfo(np.uint32).max, types.uint32),
+                                 (np.iinfo(np.uint64).max, types.uint64),
+                                 (np.finfo(np.float32).min, types.float32),
+                                 (np.finfo(np.float64).min, types.float64)
+                                 ])
+
+    @_unimplemented
+    def test_abs_ufunc_npm(self):
+        self.test_abs_ufunc(flags=no_pyobj_flags)
+
     def test_absolute_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('absolute', flags=flags,
             additional_inputs = [(np.iinfo(np.uint32).max, types.uint32),
@@ -245,13 +371,14 @@ class TestUFuncs(unittest.TestCase):
                                  (np.finfo(np.float64).min, types.float64)
                                  ])
 
+    @_unimplemented
     def test_absolute_ufunc_npm(self):
         self.test_absolute_ufunc(flags=no_pyobj_flags)
 
     def test_rint_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('rint', flags=flags)
 
-    @unittest.expectedFailure
+    @_unimplemented
     def test_rint_ufunc_npm(self):
         self.test_rint_ufunc(flags=no_pyobj_flags)
 
@@ -264,7 +391,7 @@ class TestUFuncs(unittest.TestCase):
     def test_conj_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('conj', flags=flags)
 
-    @unittest.expectedFailure
+    @_unimplemented
     def test_conj_ufunc_npm(self):
         self.test_conj_ufunc(flags=no_pyobj_flags)
 
@@ -319,17 +446,28 @@ class TestUFuncs(unittest.TestCase):
     def test_square_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('square', flags=flags)
 
-    @unittest.expectedFailure
+    @_unimplemented
     def test_square_ufunc_npm(self):
         self.test_square_ufunc(flags=no_pyobj_flags)
 
     def test_reciprocal_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('reciprocal', flags=flags)
 
-    @unittest.expectedFailure
+    @_unimplemented
     def test_reciprocal_ufunc_npm(self):
         self.test_reciprocal_ufunc(flags=no_pyobj_flags)
 
+    def test_conjugate_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('reciprocal', flags=flags)
+
+    @_unimplemented
+    def test_conjugate_ufunc_npm(self):
+        self.test_reciprocal_ufunc(flags=no_pyobj_flags)
+
+
+    ############################################################################
+    # Trigonometric Functions
+
     def test_sin_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('sin', flags=flags)
 
@@ -366,6 +504,19 @@ class TestUFuncs(unittest.TestCase):
     def test_arctan_ufunc_npm(self):
         self.test_arctan_ufunc(flags=no_pyobj_flags)
 
+    def test_arctan2_ufunc(self):
+        self.binary_ufunc_test('arctan2')
+
+    def test_arctan2_ufunc_npm(self):
+        self.binary_ufunc_test('arctan2', flags=no_pyobj_flags)
+
+    def test_hypot_ufunc(self):
+        self.binary_ufunc_test('hypot')
+
+    @_unimplemented
+    def test_hypot_ufunc_npm(self):
+        self.binary_ufunc_test('hypot', flags=no_pyobj_flags)
+
     def test_sinh_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('sinh', flags=flags)
 
@@ -414,14 +565,250 @@ class TestUFuncs(unittest.TestCase):
     def test_rad2deg_ufunc_npm(self):
         self.test_rad2deg_ufunc(flags=no_pyobj_flags)
 
-    @unittest.skipIf(not hasattr(np, "invertlogical_not"),
-                     "invertlogical_not is not available")
-    def test_invertlogical_not_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test('invertlogical_not', flags=flags)
+    def test_degrees_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('degrees', flags=flags)
 
-    @unittest.expectedFailure
-    def test_invertlogical_not_ufunc_npm(self):
-        self.test_invertlogical_not_ufunc(flags=no_pyobj_flags)
+    @_unimplemented
+    def test_degrees_ufunc_npm(self):
+        self.test_degrees_ufunc(flags=no_pyobj_flags)
+
+    def test_radians_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('radians', flags=flags)
+
+    @_unimplemented
+    def test_radians_ufunc_npm(self):
+        self.test_radians_ufunc(flags=no_pyobj_flags)
+
+
+    ############################################################################
+    # Bit-twiddling Functions
+
+    def test_bitwise_and_ufunc(self):
+        self.binary_int_ufunc_test('bitwise_and')
+
+    @_unimplemented
+    def test_bitwise_and_ufunc_npm(self):
+        self.binary_int_ufunc_test('bitwise_and', flags=no_pyobj_flags)
+
+    def test_bitwise_or_ufunc(self):
+        self.binary_int_ufunc_test('bitwise_or')
+
+    @_unimplemented
+    def test_bitwise_or_ufunc_npm(self):
+        self.binary_int_ufunc_test('bitwise_or', flags=no_pyobj_flags)
+
+    def test_bitwise_xor_ufunc(self):
+        self.binary_int_ufunc_test('bitwise_xor')
+
+    @_unimplemented
+    def test_bitwise_xor_ufunc_npm(self):
+        self.binary_int_ufunc_test('bitwise_xor', flags=no_pyobj_flags)
+
+    def test_invert_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_int_ufunc_test('invert', flags=flags)
+
+    @_unimplemented
+    def test_invert_ufunc_npm(self):
+        self.test_invert_ufunc(flags=no_pyobj_flags)
+
+    def test_bitwise_not_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_int_ufunc_test('bitwise_not', flags=flags)
+
+    @_unimplemented
+    def test_bitwise_not_ufunc_npm(self):
+        self.test_bitwise_not_ufunc(flags=no_pyobj_flags)
+
+    def test_left_shift_ufunc(self):
+        self.binary_int_ufunc_test('left_shift')
+
+    @_unimplemented
+    def test_left_shift_ufunc_npm(self):
+        self.binary_int_ufunc_test('left_shift', flags=no_pyobj_flags)
+
+    def test_right_shift_ufunc(self):
+        self.binary_int_ufunc_test('right_shift')
+
+    @_unimplemented
+    def test_right_shift_ufunc_npm(self):
+        self.binary_int_ufunc_test('right_shift', flags=no_pyobj_flags)
+
+
+    ############################################################################
+    # Comparison functions
+    def test_greater_ufunc(self):
+        self.binary_ufunc_test('greater')
+
+    @_unimplemented
+    def test_greater_ufunc_npm(self):
+        self.binary_ufunc_test('greater', flags=no_pyobj_flags)
+
+    def test_greater_equal_ufunc(self):
+        self.binary_ufunc_test('greater_equal')
+
+    @_unimplemented
+    def test_greater_equal_ufunc_npm(self):
+        self.binary_ufunc_test('greater_equal', flags=no_pyobj_flags)
+
+    def test_less_ufunc(self):
+        self.binary_ufunc_test('less')
+
+    @_unimplemented
+    def test_less_ufunc_npm(self):
+        self.binary_ufunc_test('less', flags=no_pyobj_flags)
+
+    def test_less_equal_ufunc(self):
+        self.binary_ufunc_test('less_equal')
+
+    @_unimplemented
+    def test_less_equal_ufunc_npm(self):
+        self.binary_ufunc_test('less_equal', flags=no_pyobj_flags)
+
+    def test_not_equal_ufunc(self):
+        self.binary_ufunc_test('not_equal')
+
+    @_unimplemented
+    def test_not_equal_ufunc_npm(self):
+        self.binary_ufunc_test('not_equal', flags=no_pyobj_flags)
+
+    def test_equal_ufunc(self):
+        self.binary_ufunc_test('equal')
+
+    @_unimplemented
+    def test_equal_ufunc_npm(self):
+        self.binary_ufunc_test('equal', flags=no_pyobj_flags)
+
+    def test_logical_and_ufunc(self):
+        self.binary_ufunc_test('logical_and')
+
+    @_unimplemented
+    def test_logical_and_ufunc_npm(self):
+        self.binary_ufunc_test('logical_and', flags=no_pyobj_flags)
+
+    def test_logical_or_ufunc(self):
+        self.binary_ufunc_test('logical_or')
+
+    @_unimplemented
+    def test_logical_or_ufunc_npm(self):
+        self.binary_ufunc_test('logical_or', flags=no_pyobj_flags)
+
+    def test_logical_xor_ufunc(self):
+        self.binary_ufunc_test('logical_xor')
+
+    @_unimplemented
+    def test_logical_xor_ufunc_npm(self):
+        self.binary_ufunc_test('logical_xor', flags=no_pyobj_flags)
+
+    #logical_not support should be coming soon
+    def test_logical_not_ufunc(self):
+        self.unary_ufunc_test('logical_not')
+
+    @_unimplemented
+    def test_logical_not_ufunc_npm(self):
+        self.unary_ufunc_test('logical_not', flags=no_pyobj_flags)
+
+    def test_maximum_ufunc(self):
+        self.binary_ufunc_test('maximum')
+
+    @_unimplemented
+    def test_maximum_ufunc_npm(self):
+        self.binary_ufunc_test('maximum', flags=no_pyobj_flags)
+
+    def test_minimum_ufunc(self):
+        self.binary_ufunc_test('minimum')
+
+    @_unimplemented
+    def test_minimum_ufunc_npm(self):
+        self.binary_ufunc_test('minimum', flags=no_pyobj_flags)
+
+    def test_fmax_ufunc(self):
+        self.binary_ufunc_test('fmax')
+    
+    @_unimplemented
+    def test_fmax_ufunc_npm(self):
+        self.binary_ufunc_test('fmax', flags=no_pyobj_flags)
+
+    def test_fmin_ufunc(self):
+        self.binary_ufunc_test('fmin')
+
+    @_unimplemented
+    def test_fmin_ufunc_npm(self):
+        self.binary_ufunc_test('fmin', flags=no_pyobj_flags)
+
+
+    ############################################################################
+    # Floating functions
+    def test_isfinite_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('isfinite', flags=flags)
+
+    @_unimplemented
+    def test_isfinite_ufunc_npm(self):
+        self.test_isfinite_ufunc(flags=no_pyobj_flags)
+
+    def test_isinf_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('isinf', flags=flags)
+
+    @_unimplemented
+    def test_isinf_ufunc_npm(self):
+        self.test_isinf_ufunc(flags=no_pyobj_flags)
+
+    def test_isnan_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('isnan', flags=flags)
+
+    @_unimplemented
+    def test_isnan_ufunc_npm(self):
+        self.test_isnan_ufunc(flags=no_pyobj_flags)
+
+    def test_signbit_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('signbit', flags=flags)
+
+    @_unimplemented
+    def test_signbit_ufunc_npm(self):
+        self.test_signbit_ufunc(flags=no_pyobj_flags)
+
+    def test_copysign_ufunc(self, flags=enable_pyobj_flags):
+        self.binary_ufunc_test('copysign')
+
+    def test_copysign_ufunc_npm(self):
+        self.test_copysign_ufunc(flags=no_pyobj_flags)
+
+    @_unimplemented
+    def test_nextafter_ufunc(self, flags=enable_pyobj_flags):
+        self.binary_ufunc_test('nextafter', flags=flags)
+
+    @_unimplemented
+    def test_nextafter_ufunc_npm(self):
+        self.test_nextafter_ufunc(flags=no_pyobj_flags)
+
+    def test_modf_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('modf', flags=flags)
+
+    @_unimplemented
+    def test_modf_ufunc_npm(self):
+        self.test_modf_ufunc(flags=no_pyobj_flags)
+
+
+    # FIXME - ldexp does not have homogeneous arguments, so the usual tests won't
+    #         work as they reuse both inputs
+    @unittest.skipIf(is32bits or iswindows, "Some types are not supported on "
+                                       "32-bit "
+                               "platform")
+    @unittest.skip("this test needs fixing")
+    def test_ldexp_ufunc(self):
+        self.binary_int_ufunc_test('ldexp')
+
+    # FIXME
+    @unittest.skipIf(is32bits or iswindows,
+                     "Some types are not supported on 32-bit platform")
+    @unittest.skip("this tests needs fixing")
+    def test_ldexp_ufunc_npm(self):
+        self.binary_int_ufunc_test('ldexp', flags=no_pyobj_flags)
+
+    def test_frexp_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('frexp', flags=flags)
+
+    @_unimplemented
+    def test_frexp_ufunc_npm(self):
+        self.test_frexp_ufunc(flags=no_pyobj_flags)
 
     def test_floor_ufunc(self, flags=enable_pyobj_flags):
         self.unary_ufunc_test('floor', flags=flags)
@@ -441,257 +828,15 @@ class TestUFuncs(unittest.TestCase):
     def test_trunc_ufunc_npm(self):
         self.test_trunc_ufunc(flags=no_pyobj_flags)
 
-    # binary ufunc tests
-    def test_add_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test('add', flags=flags)
+    def test_spacing_ufunc(self, flags=enable_pyobj_flags):
+        self.unary_ufunc_test('spacing', flags=flags)
 
-    def test_add_ufunc_npm(self):
-        self.test_add_ufunc(flags=no_pyobj_flags)
-
-    def test_subtract_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test('subtract', flags=flags)
-
-    def test_subtract_ufunc_npm(self):
-        self.test_subtract_ufunc(flags=no_pyobj_flags)
-
-    def test_multiply_ufunc(self, flags=enable_pyobj_flags):
-        self.binary_ufunc_test('multiply', flags=flags)
-
-    def test_multiply_ufunc_npm(self):
-        self.test_multiply_ufunc(flags=no_pyobj_flags)
-
-    def test_divide_ufunc(self, flags=enable_pyobj_flags):
-        skip_inputs = []
-        # python3 integer division by zero and
-        # storing in 64 bit int produces garbage
-        # instead of 0, so skip
-        if PYVERSION >= (3, 0):
-            skip_inputs = [types.uint32, types.uint64,
-                           types.Array(types.uint32, 1, 'C'),
-                           types.Array(types.int32, 1, 'C'),
-                           types.Array(types.uint64, 1, 'C')]
-        self.binary_ufunc_test('divide', flags=flags,
-            skip_inputs=skip_inputs, int_output_type=types.float64)
-
-    def test_divide_ufunc_npm(self):
-        self.test_divide_ufunc(flags=no_pyobj_flags)
-
-    def test_logaddexp_ufunc(self):
-        self.binary_ufunc_test('logaddexp')
-
-    @unittest.expectedFailure
-    def test_logaddexp_ufunc_npm(self):
-        self.binary_ufunc_test('logaddexp', flags=no_pyobj_flags)
-
-    def test_logaddexp2_ufunc(self):
-        self.binary_ufunc_test('logaddexp2')
-
-    @unittest.expectedFailure
-    def test_logaddexp2_ufunc_npm(self):
-        self.binary_ufunc_test('logaddexp2', flags=no_pyobj_flags)
-
-    def test_true_divide_ufunc(self):
-        self.binary_ufunc_test('true_divide')
-
-    @unittest.expectedFailure
-    def test_true_divide_ufunc_npm(self):
-        self.binary_ufunc_test('true_divide', flags=no_pyobj_flags)
-
-    def test_floor_divide_ufunc(self):
-        self.binary_ufunc_test('floor_divide')
-
-    @unittest.expectedFailure
-    def test_floor_divide_ufunc_npm(self):
-        self.binary_ufunc_test('floor_divide', flags=no_pyobj_flags)
-
-    def test_power_ufunc(self):
-        self.binary_ufunc_test('power')
-
-    def test_power_ufunc_npm(self):
-        self.binary_ufunc_test('power', flags=no_pyobj_flags)
-
-    def test_remainder_ufunc(self):
-        self.binary_ufunc_test('remainder')
-
-    @unittest.expectedFailure
-    def test_remainder_ufunc_npm(self):
-        self.binary_ufunc_test('remainder', flags=no_pyobj_flags)
-
-    def test_mod_ufunc(self):
-        self.binary_ufunc_test('mod')
-
-    @unittest.expectedFailure
-    def test_mod_ufunc_npm(self):
-        self.binary_ufunc_test('mod', flags=no_pyobj_flags)
-
-    def test_fmod_ufunc(self):
-        self.binary_ufunc_test('fmod')
-
-    @unittest.expectedFailure
-    def test_fmod_ufunc_npm(self):
-        self.binary_ufunc_test('fmod', flags=no_pyobj_flags)
-
-    def test_arctan2_ufunc(self):
-        self.binary_ufunc_test('arctan2')
-
-    def test_arctan2_ufunc_npm(self):
-        self.binary_ufunc_test('arctan2', flags=no_pyobj_flags)
-
-    def test_hypot_ufunc(self):
-        self.binary_ufunc_test('hypot')
-
-    @unittest.expectedFailure
-    def test_hypot_ufunc_npm(self):
-        self.binary_ufunc_test('hypot', flags=no_pyobj_flags)
-
-    def test_bitwise_and_ufunc(self):
-        self.binary_int_ufunc_test('bitwise_and')
-
-    @unittest.expectedFailure
-    def test_bitwise_and_ufunc_npm(self):
-        self.binary_int_ufunc_test('bitwise_and', flags=no_pyobj_flags)
-
-    def test_bitwise_or_ufunc(self):
-        self.binary_int_ufunc_test('bitwise_or')
-
-    @unittest.expectedFailure
-    def test_bitwise_or_ufunc_npm(self):
-        self.binary_int_ufunc_test('bitwise_or', flags=no_pyobj_flags)
-
-    def test_bitwise_xor_ufunc(self):
-        self.binary_int_ufunc_test('bitwise_xor')
-
-    @unittest.expectedFailure
-    def test_bitwise_xor_ufunc_npm(self):
-        self.binary_int_ufunc_test('bitwise_xor', flags=no_pyobj_flags)
-
-    def test_left_shift_ufunc(self):
-        self.binary_int_ufunc_test('left_shift')
-
-    @unittest.expectedFailure
-    def test_left_shift_ufunc_npm(self):
-        self.binary_int_ufunc_test('left_shift', flags=no_pyobj_flags)
-
-    def test_right_shift_ufunc(self):
-        self.binary_int_ufunc_test('right_shift')
-
-    @unittest.expectedFailure
-    def test_right_shift_ufunc_npm(self):
-        self.binary_int_ufunc_test('right_shift', flags=no_pyobj_flags)
-
-    def test_greater_ufunc(self):
-        self.binary_ufunc_test('greater')
-
-    @unittest.expectedFailure
-    def test_greater_ufunc_npm(self):
-        self.binary_ufunc_test('greater', flags=no_pyobj_flags)
-
-    def test_greater_equal_ufunc(self):
-        self.binary_ufunc_test('greater_equal')
-
-    @unittest.expectedFailure
-    def test_greater_equal_ufunc_npm(self):
-        self.binary_ufunc_test('greater_equal', flags=no_pyobj_flags)
-
-    def test_less_ufunc(self):
-        self.binary_ufunc_test('less')
-
-    @unittest.expectedFailure
-    def test_less_ufunc_npm(self):
-        self.binary_ufunc_test('less', flags=no_pyobj_flags)
-
-    def test_less_equal_ufunc(self):
-        self.binary_ufunc_test('less_equal')
-
-    @unittest.expectedFailure
-    def test_less_equal_ufunc_npm(self):
-        self.binary_ufunc_test('less_equal', flags=no_pyobj_flags)
-
-    def test_not_equal_ufunc(self):
-        self.binary_ufunc_test('not_equal')
-
-    @unittest.expectedFailure
-    def test_not_equal_ufunc_npm(self):
-        self.binary_ufunc_test('not_equal', flags=no_pyobj_flags)
-
-    def test_equal_ufunc(self):
-        self.binary_ufunc_test('equal')
-
-    @unittest.expectedFailure
-    def test_equal_ufunc_npm(self):
-        self.binary_ufunc_test('equal', flags=no_pyobj_flags)
-
-    def test_logical_and_ufunc(self):
-        self.binary_ufunc_test('logical_and')
-
-    @unittest.expectedFailure
-    def test_logical_and_ufunc_npm(self):
-        self.binary_ufunc_test('logical_and', flags=no_pyobj_flags)
-
-    def test_logical_or_ufunc(self):
-        self.binary_ufunc_test('logical_or')
-
-    @unittest.expectedFailure
-    def test_logical_or_ufunc_npm(self):
-        self.binary_ufunc_test('logical_or', flags=no_pyobj_flags)
-
-    def test_logical_xor_ufunc(self):
-        self.binary_ufunc_test('logical_xor')
-
-    @unittest.expectedFailure
-    def test_logical_xor_ufunc_npm(self):
-        self.binary_ufunc_test('logical_xor', flags=no_pyobj_flags)
-
-    def test_maximum_ufunc(self):
-        self.binary_ufunc_test('maximum')
-
-    @unittest.expectedFailure
-    def test_maximum_ufunc_npm(self):
-        self.binary_ufunc_test('maximum', flags=no_pyobj_flags)
-
-    def test_minimum_ufunc(self):
-        self.binary_ufunc_test('minimum')
-
-    @unittest.expectedFailure
-    def test_minimum_ufunc_npm(self):
-        self.binary_ufunc_test('minimum', flags=no_pyobj_flags)
-
-    def test_fmax_ufunc(self):
-        self.binary_ufunc_test('fmax')
-
-    @unittest.expectedFailure
-    def test_fmax_ufunc_npm(self):
-        self.binary_ufunc_test('fmax', flags=no_pyobj_flags)
-
-    def test_fmin_ufunc(self):
-        self.binary_ufunc_test('fmin')
-
-    @unittest.expectedFailure
-    def test_fmin_ufunc_npm(self):
-        self.binary_ufunc_test('fmin', flags=no_pyobj_flags)
-
-    def test_copysign_ufunc(self):
-        self.binary_ufunc_test('copysign')
-
-    @unittest.expectedFailure
-    def test_copysign_ufunc_npm(self):
-        self.binary_ufunc_test('copysign', flags=no_pyobj_flags)
-
-    # FIXME
-    @unittest.skipIf(is32bits or iswindows, "Some types are not supported on "
-                                       "32-bit "
-                               "platform")
-    @unittest.expectedFailure
-    def test_ldexp_ufunc(self):
-        self.binary_int_ufunc_test('ldexp')
-
-    # FIXME
-    @unittest.skipIf(is32bits or iswindows,
-                     "Some types are not supported on 32-bit platform")
-    @unittest.expectedFailure
-    def test_ldexp_ufunc_npm(self):
-        self.binary_int_ufunc_test('ldexp', flags=no_pyobj_flags)
+    @_unimplemented
+    def test_spacing_ufunc_npm(self):
+        self.test_spacing_ufunc(flags=nopyobj_flags)
 
+    ############################################################################
+    # Other tests
     def test_binary_ufunc_performance(self):
 
         pyfunc = _make_binary_ufunc_usecase('add')
@@ -761,8 +906,9 @@ class TestUFuncs(unittest.TestCase):
                     not isinstance(output_type, types.Array)):
                 continue
 
-            cr = compile_isolated(pyfunc, (input1_type, input2_type, output_type),
-                                  flags=flags)
+            cr = self.cache.compile(pyfunc,
+                                    (input1_type, input2_type, output_type),
+                                    flags=flags)
             cfunc = cr.entry_point
 
             if isinstance(input1_operand, np.ndarray):
@@ -822,8 +968,8 @@ class TestUFuncs(unittest.TestCase):
             input_type = types.Array(types.uint64, x.ndim, 'C')
             output_type = types.Array(types.int64, result.ndim, 'C')
 
-            cr = compile_isolated(pyfunc, (input_type, output_type),
-                                  flags=no_pyobj_flags)
+            cr = self.cache.compile(pyfunc, (input_type, output_type),
+                                    flags=no_pyobj_flags)
             cfunc = cr.entry_point
 
             expected = np.zeros(result.shape, dtype=result.dtype)
@@ -854,8 +1000,8 @@ class TestUFuncs(unittest.TestCase):
             input2_type = types.Array(types.uint64, y.ndim, 'C')
             output_type = types.Array(types.uint64, max(x.ndim, y.ndim), 'C')
 
-            cr = compile_isolated(pyfunc, (input1_type, input2_type, output_type),
-                                  flags=no_pyobj_flags)
+            cr = self.cache.compile(pyfunc, (input1_type, input2_type, output_type),
+                                    flags=no_pyobj_flags)
             cfunc = cr.entry_point
 
             expected = np.add(x, y)
